/** This class handles all GUI interactions.
 *
 *  @author Julian Strohmayer 01426125 <e1426125@student.tuwien.ac.at>
 *  @author Patrick Gantner 01576033 <e1576033@student.tuwien.ac.at>
 *  @date 05.06.2018
 */

#include "MainWindow.h"
#include <QFileDialog>
#include <QPainter>
#include <QMessageBox>

#include "Renderable.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), volume(0)
{
    mainWindow = new Ui::MainWindow();
    mainWindow->setupUi(this);

    connect(mainWindow->actionOpen, SIGNAL(triggered()), this, SLOT(openFileAction()));
    connect(mainWindow->actionClose, SIGNAL(triggered()), this, SLOT(closeAction()));
}

MainWindow::~MainWindow()
{
    delete volume;
}

void MainWindow::on_renderButton_clicked()
{
    if(readyToDraw){
            // pass transfer function parameters
            mainWindow->AlphaCompositingOpenGLWidget->setRegion(regionSeperator,regionColor1,regionColor2);
            mainWindow->HuePreservingOpenGLWidget->setRegion(regionSeperator,regionColor1,regionColor2);

            // render full screen quad on both widgets
            std::vector<glm::vec3> vertices = { glm::vec3(-1.0f, -1.0f, 0.0f), glm::vec3(1.0f, -1.0f, 0.0f), glm::vec3(-1.0f, 1.0f, 0.0f),glm::vec3(1.0f, 1.0f, 0.0f) };
            std::vector<unsigned int> indices = { 0, 1, 2, 1, 3, 2};
            RenderData data = { vertices, indices };
            mainWindow->AlphaCompositingOpenGLWidget->setShader("shader/passthrough.vert", "shader/alpha.frag");
            mainWindow->AlphaCompositingOpenGLWidget->setRenderObject(data);
            mainWindow->AlphaCompositingOpenGLWidget->setOrientation(mainWindow->orientationBox->currentIndex());
            mainWindow->AlphaCompositingOpenGLWidget->update();
            mainWindow->HuePreservingOpenGLWidget->setShader("shader/passthrough.vert", "shader/hue.frag");
            mainWindow->HuePreservingOpenGLWidget->setRenderObject(data);
            mainWindow->HuePreservingOpenGLWidget->setOrientation(mainWindow->orientationBox->currentIndex());
            mainWindow->HuePreservingOpenGLWidget->update();
    }
    else
    {
        QMessageBox msgBox;
        msgBox.setText("Please select a data set to render!");
        msgBox.exec();
    }
}

void MainWindow::openFileAction()
{
	QString filename = QFileDialog::getOpenFileName(this, "Data File", 0, tr("Data Files (*.dat *.gri *.csv)"));

	if (!filename.isEmpty()){
        string fn = filename.toStdString();
		bool success = false;
        mainWindow->progressBar->setEnabled(true);
        mainWindow->labelTop->setText("Loading data ...");


        if (fn.substr(fn.find_last_of(".") + 1) == "dat"){
            volume = new Volume();
            success = volume->loadFromFile(filename,mainWindow->progressBar);
		}

        mainWindow->progressBar->setEnabled(false);

        if (success) {
            auto tex = volume->generate3DTex();
            if (tex->isCreated()) {
                // set volume
                mainWindow->AlphaCompositingOpenGLWidget->setVolumeTexture(tex);
                mainWindow->HuePreservingOpenGLWidget->setVolumeTexture(tex);
            }
        }

		if (success){
            mainWindow->labelTop->setText("File LOADED [" + filename + "]");
			readyToDraw = true;
        }
        else{
            mainWindow->labelTop->setText("ERROR loading file " + filename + "!");
            mainWindow->progressBar->setValue(0);
        }
    }
}

void MainWindow::closeAction()
{
	close();
}

void MainWindow::on_color1Button_clicked()
{
   QColor color = QColorDialog::getColor(Qt::red,this,"Choose a color for region 1");
   if(color.isValid()){
       QString qss = QString("background-color: %1").arg(color.name());
       mainWindow->color1Button->setStyleSheet(qss);
       regionColor1 = color;
       updateBothViews();
   }
}

void MainWindow::on_color2Button_clicked()
{
    QColor color = QColorDialog::getColor(Qt::red,this,"Choose a color for region 2");
    if(color.isValid()){
        QString qss = QString("background-color: %1").arg(color.name());
        mainWindow->color2Button->setStyleSheet(qss);
         regionColor2 = color;
         updateBothViews();
    }
}

void MainWindow::on_region1Slider_valueChanged()
{
    float val = mainWindow->region1Slider->value();
    QString string = "[0,";
    string.append(QString::number(val/100));
    string.append(")");
    mainWindow->region1RangeLabel->setText(string);

    string = "[";
    string.append(QString::number(val/100));
    string.append(",1]");
    mainWindow->region2RangeLabel->setText(string);
    regionSeperator = val/100;
}

void MainWindow::on_region1Slider_sliderReleased(){
   updateBothViews();
}

void MainWindow::updateBothViews(){
    mainWindow->AlphaCompositingOpenGLWidget->setRegion(regionSeperator,regionColor1,regionColor2);
    mainWindow->HuePreservingOpenGLWidget->setRegion(regionSeperator,regionColor1,regionColor2);
    mainWindow->AlphaCompositingOpenGLWidget->update();
    mainWindow->HuePreservingOpenGLWidget->update();
}

void MainWindow::on_orientationBox_currentIndexChanged(int index)
{
    mainWindow->AlphaCompositingOpenGLWidget->setOrientation(index);
    mainWindow->HuePreservingOpenGLWidget->setOrientation(index);
    updateBothViews();
}
