/** This class handles all GUI interactions.
 *
 *  @author Julian Strohmayer 01426125 <e1426125@student.tuwien.ac.at>
 *  @author Patrick Gantner 01576033 <e1576033@student.tuwien.ac.at>
 *  @date 05.06.2018
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <stdio.h>
#include <stdlib.h>

#include "ui_MainWindow.h"
#include "Volume.h"

#include <QMainWindow>
#include <QPushButton>
#include <QLabel>
#include <QProgressBar>
#include <QStatusBar>
#include <QVariant>
#include <QColorDialog>
#include <QColor>

using namespace std;

/**
 * @brief The MainWindow class handliung GUI interactions.
 */
class MainWindow : public QMainWindow
{
	Q_OBJECT

	public:
		MainWindow(QWidget *parent = 0);
		~MainWindow();

	protected slots :      
        /**
         * @brief openFileAction opens a file-chooser dialogue whenever the "Open..." option in the drop down menu is selected.
         * The selected volume data set is loaded into memory and will be rendered once
         * all transfer function parameters have been set and the "start render" button is clicked.
         */
		void openFileAction();

        /**
         * @brief closeAction terminates the program whenever the "Close" option in the drop down menu is selected.
         */
		void closeAction();

    private slots:

        /**
         * @brief on_renderButton_clicked initates the rendering of the volume data sets with both methods.
         * The results are displayed in the corresponding GUI windows.
         * @pre A data set must be loaded and both transfer function colors have to be set otherwise noting will be rendered.
         */
        void on_renderButton_clicked();

        /**
         * @brief on_color1Button_clicked opens a color-picker dialogue for the lower end (low densisty) of the transfer function.
         * If the "start rendering" button had been already pressed beforerhand and the a volume was rendered successfully
         * then both views are automatically updated and the volume is rendered with the new color.
         */
        void on_color1Button_clicked();

        /**
         * @brief on_color2Button_clicked opens a color-picker dialogue for the upper end (high densisty) of the transfer function.
         * If the "start rendering" button had been already pressed beforerhand and the a volume was rendered successfully
         * then both views are automatically updated and the volume is rendered with the new color.
         */
        void on_color2Button_clicked();

        /**
         * @brief on_region1Slider_valueChanged sets the boundary value between the regions of the transfer function (default=0.5).
         * Both regions intervalls are displayed in the UI next to both ends of the slider.
         */
        void on_region1Slider_valueChanged();

        /**
         * @brief on_region1Slider_sliderReleased automatically updates both views and the volume is rendered with the new boundary if
         * the "start rendering" button had been already pressed beforerhand and the volume was rendered successfully.
         */
        void on_region1Slider_sliderReleased();

        /**
         * @brief updateBothViews passes both region colors and the region boundary value to the shaders.
         * It is triggered by on_color1Button_clicked, on_color2Button_clicked and on_region1Slider_sliderReleased
         */
        void updateBothViews();

        void on_orientationBox_currentIndexChanged(int index);

private:
        // User Interface Elements
        Ui::MainWindow *mainWindow;

        // File loaded successfully
		bool readyToDraw = false;
		
        // Volume Data
        Volume *volume;

        // Region seperator value (0,1)
        float regionSeperator=0.5;

        // Region colors;
        QColor regionColor1;
        QColor regionColor2;
};

#endif
