/**
 *  @author Julian Strohmayer 01426125 <e1426125@student.tuwien.ac.at>
 *  @author Patrick Gantner 01576033 <e1576033@student.tuwien.ac.at>
 *  @date 05.06.2018
 */

#pragma once

#include <vector>
#include <string>
#include <iostream>

#include <QProgressBar>

#include <QOpenGLTexture>


//-------------------------------------------------------------------------------------------------
// Voxel
//-------------------------------------------------------------------------------------------------

/**
 * @brief Helper class for Voxels with basic operators.
 */
class Voxel
{
	public:

		Voxel();
		Voxel(const Voxel &other);
		Voxel(const float value);

		~Voxel();


		// VOXEL VALUE

		void					setValue(const float value);
		const float				getValue() const;


		// OPERATORS

		const bool				operator==(const Voxel &other) const;
		const bool				operator!=(const Voxel &other) const;
		const bool				operator>(const Voxel &other) const;
		const bool				operator>=(const Voxel &other) const;
		const bool				operator<(const Voxel &other) const;
		const bool				operator<=(const Voxel &other) const;

		const Voxel				operator+(const Voxel &other) const;
		const Voxel				operator-(const Voxel &other) const;
		const Voxel				operator*(const float &value) const;
		const Voxel				operator/(const float &value) const;
		
		const Voxel&			operator+=(const Voxel &other);
		const Voxel&			operator-=(const Voxel &other);
		const Voxel&			operator*=(const float &value);
		const Voxel&			operator/=(const float &value);


	private:

		float					m_Value;

};


//-------------------------------------------------------------------------------------------------
// Volume
//-------------------------------------------------------------------------------------------------

/**
 * @brief Modified volume loader class from Vis 1.
 */
class Volume
{

	public:

		Volume();
		~Volume();


		// VOLUME DATA

		const Voxel&			voxel(const int i) const;
		const Voxel&			voxel(const int x, const int y, const int z) const;
		const Voxel*			voxels() const;

		const int				width() const;
		const int				height() const;
		const int				depth() const;

		const int				size() const;

		bool					loadFromFile(QString filename, QProgressBar* progressBar);

        /**
         * @brief generate3DTex transforms the loaded volume data set into a 3D texture which can
         * be sampled in the glsl shaders.
         * @return volume data set as 3D texture pointer
         */
        QOpenGLTexture*          generate3DTex();

	private:

		std::vector<Voxel>		m_Voxels;

		int						m_Width;
		int						m_Height;
		int						m_Depth;

		int						m_Size;

};
