/**
* @file alpha.frag
* @class AlphaCompositingShader
* @brief This fragment shader performs front-to-back alpha compositing with early ray termination.
* @author Julian Strohmayer 01426125 <e1426125@student.tuwien.ac.at>
* @author Patrick Gantner 01576033 <e1576033@student.tuwien.ac.at>
* @date 05.06.2018
*/

#version 400

out vec4 color;

in vec3 position;

uniform sampler3D volume;
uniform float regionSeperator;
uniform int depth;
uniform int width;
uniform int height;
uniform vec4 regionColor1;
uniform vec4 regionColor2;
uniform int orientation;

void main() {

    // transform normalized device coordinates [-1,-1] to volume coordinate system [0,1]
    vec3 volPosition = (position+1)/2;

    float alphaB; // back voxel
    vec4 colorB;

    float x;
    float y;
    float z;

    // Front-to-Back compositing
    for(float i=0;i<depth;i++)
    {
        switch (orientation) {
            default:
            case 0: // top
                x = volPosition.x;
                y = volPosition.y;
                z = i/depth;
                break;
            case 1: // bottom
                x = 1.0f - volPosition.x;
                y = 1.0f - volPosition.y;
                z = (depth - i - 1)/depth;
                break;
            case 2: // left
                x = i/depth;
                y = volPosition.x;
                z = 1.0f - volPosition.y;
                break;
            case 3: // right
                x = (depth - i - 1) / depth;
                y = 1 - volPosition.x;
                z = 1.0f - volPosition.y;
                break;
            case 4: // front
                x = volPosition.x;
                y = i/depth;
                z = 1.0f - volPosition.y;
                break;
            case 5: // back
                x = 1 - volPosition.x;
                y = (depth - i - 1) / depth;
                z = 1.0f - volPosition.y;
                break;
        }
        colorB.a = texture(volume,vec3(x,y,z)).r;
        colorB.rgb = (colorB.a<regionSeperator)?regionColor1.rgb:regionColor2.rgb;
        color.rgb = color.rgb + (1-color.a)*colorB.rgb*colorB.a;
        color.a = min(1,color.a + (1-color.a)*colorB.a*colorB.a);

        if(color.a==1){break;} // early ray termination
    }

    }


